<?php
/*
 * This file is part of FacetWP Carousel.
 *
 * FacetWP Carousel is free software: you can redistribute it and/or
 * modify it under the terms of the GNU Affero General Public
 * License as published by the Free Software Foundation, either
 * version 3 of the License, or (at your option) any later version.
 *
 * FacetWP Carousel is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with FacetWP Carousel.  If not, see <https://www.gnu.org/licenses/>.
 */

declare( strict_types=1 );

namespace FacetWPCarousel\Carousel;

use WP_Query;
use function FWP;

class Shortcode {
	private string $plugin_file;

	/**
	 * @param string $plugin_file
	 */
	public function __construct( string $plugin_file ) {
		$this->plugin_file = $plugin_file;
		add_action( 'wp_enqueue_scripts', [ $this, 'register_assets' ] );
		add_shortcode( 'facetwp_carousel', [ $this, 'render' ] );
	}

	public function register_assets() {
		/* @noinspection PhpIncludeInspection */
		$asset = include plugin_dir_path( $this->plugin_file ) . 'build/carousel.asset.php';

		wp_register_style(
			'facetwp-carousel',
			plugins_url( 'build/carousel.css', $this->plugin_file ),
			[],
			$asset['version']
		);

		wp_register_script(
			'facetwp-carousel',
			plugins_url( 'build/carousel.js', $this->plugin_file ),
			$asset['dependencies'],
			$asset['version'],
			true
		);
	}

	/**
	 * @param array $atts
	 *
	 * @return string
	 */
	public function render( array $atts ): string {
		$atts = shortcode_atts(
			[
				'template' => '',
				'post_type' => 'product',
				'taxonomy' => 'product_cat',
				'category' => '',
				'limit' => 10,
				'slides_desktop' => 4,
				'slides_tablet' => 3,
				'slides_mobile' => 2,
				'featured_products' => false,
				'product_upsells' => false,
				'product_related' => false,
				'arrows' => true,
				'dots' => false,
				'use_template_query' => false,
			],
			$atts
		);
		$template = FWP()->helper->get_template_by_name( $atts['template'] );
		if ( ! $template ) {
			return '';
		}

		wp_enqueue_style( 'facetwp-carousel' );
		wp_enqueue_script( 'facetwp-carousel' );

		if ( $atts['use_template_query'] ) {
			$args = $template['query_obj'];
		} else {
			$args = $this->buildQueryArgs(
				$atts['post_type'],
				$atts['taxonomy'],
				$atts['category'],
				(int) $atts['limit'],
				! ! $atts['featured_products'],
				! ! $atts['product_upsells'],
				! ! $atts['product_related']
			);
		}
		$rendered = $this->renderLayout( $template['layout'], $args );
		$output = $this->renderCarousel(
			$atts['template'],
			$rendered,
			absint( $atts['slides_desktop'] ),
			absint( $atts['slides_tablet'] ),
			absint( $atts['slides_mobile'] ),
			! ! $atts['arrows'],
			! ! $atts['dots']
		);

		/*
		 * The filter is just to perform the same actions as the vanilla
		 * facetwp actions; do not know what is it for.
		 */

		return apply_filters( 'facetwp_shortcode_html', $output, $atts );
	}

	/**
	 * Builds the query arguments to pass to FacetWP.
	 *
	 * @param string $post_type The type of posts to show.
	 * @param string $taxonomy The taxonomy to use to filter posts by category.
	 * @param string $category The actual taxonomy term for the category.
	 * @param int $limit The maximum number of posts to show.
	 * @param bool $featured_products Show only featured products.
	 * @param bool $product_upsells Show only products that are upsells of the current product.
	 * @param bool $product_related Show only products related to the current product.
	 *
	 * @return array The arguments to WP_Query to use for FacetWP.
	 *
	 * @since 20210907
	 */
	private function buildQueryArgs(
		string $post_type,
		string $taxonomy,
		string $category,
		int $limit,
		bool $featured_products,
		bool $product_upsells,
		bool $product_related
	): array {
		global $product;

		$args = [
			'post_type' => $post_type,
			'posts_per_page' => $limit,
			'post_status' => [ 'publish' ],
		];
		$tax_query = [];
		if ( $product_upsells && $product ) {
			// If get_upsell_ids returns empty array, the zero forces to match no product.
			$args['post__in'] = [ 0, ...$product->get_upsell_ids() ];
		}
		if ( $product_related && $product ) {
			// If wc_get_related_products returns empty array, the zero forces to match no product.
			$args['post__in'] = [
				0,
				...wc_get_related_products( $product->get_id(), $limit, $product->get_upsell_ids() )
			];
		}
		if ( strlen( $taxonomy ) > 0 && strlen( $category ) > 0 ) {
			$tax_query[] = [
				'taxonomy' => $taxonomy,
				'field' => 'slug',
				'terms' => $category,
			];
		}
		if ( $featured_products ) {
			$tax_query[] = [
				'taxonomy' => 'product_visibility',
				'field' => 'name',
				'terms' => 'featured',
				'operator' => 'IN',
			];
		}
		if ( ! empty( $tax_query ) ) {
			$args['tax_query'] = $tax_query;
		}

		return $args;
	}

	/**
	 * @param $layout
	 *
	 * @return string
	 */
	private function renderLayout( array $layout, array $args ): string {
		/*
		 * FacetWP’s builder uses the global $wp_query to retrieve the list of
		 * posts to show, and $post to render its layout, hence we keep a copy
		 * of the current query and post and then restore them again once we
		 * are done.
		 */
		global $wp_query;
		global $post;
		$temp_query = is_object( $wp_query ) ? clone $wp_query : $wp_query;
		$temp_post = is_object( $post ) ? clone $post : $post;

		$wp_query = new WP_Query( $args ); // phpcs:ignore WordPress.WP.GlobalVariablesOverride
		$rendered = FWP()->builder->render_layout( $layout );

		$post = $temp_post; // phpcs:ignore WordPress.WP.GlobalVariablesOverride
		$wp_query = $temp_query; // phpcs:ignore WordPress.WP.GlobalVariablesOverride
		wp_reset_postdata();

		return $rendered;
	}

	/**
	 * Renders the whole carousel.
	 *
	 * @param string $template_name The name of the template being rendered as a carousel.
	 * @param string $rendered The HTML of the rendered template.
	 * @param int $desktop_slides The number of slides to show in desktop.
	 * @param int $tablet_slides The number of slides to show in a tablet.
	 * @param int $mobile_slides The number of slides to show in a mobile.
	 * @param bool $show_arrows Whether to show the next and previous arrows.
	 * @param bool $show_dots Whether to show the dots below the slides.
	 *
	 * @return string The HTML for the whole carousel, including arrows and data attributes.
	 *
	 * @since 20210907
	 */
	private function renderCarousel(
		string $template_name,
		string $rendered,
		int $desktop_slides,
		int $tablet_slides,
		int $mobile_slides,
		bool $show_arrows,
		bool $show_dots
	): string {
		$next_arrow = file_get_contents( plugin_dir_path( $this->plugin_file ) . 'src/arrow-right.svg' );
		$prev_arrow = file_get_contents( plugin_dir_path( $this->plugin_file ) . 'src/arrow-left.svg' );

		return sprintf(
			'
<div class="facetwp-carousel" data-name="%s" data-slides-desktop="%d" data-slides-tablet="%d" data-slides-mobile="%d" %s %s>
	<a class="slick-prev">%s</a>
	%s
	<a class="slick-next">%s</a>
</div>
',
			esc_attr( $template_name ),
			$desktop_slides,
			$tablet_slides,
			$mobile_slides,
			$show_dots ? 'data-show-dots="show"' : '',
			$show_arrows ? 'data-show-arrows="show"' : '',
			$show_arrows ? str_replace( ':alt', __( 'Previous', 'facetwp-carousel' ), $prev_arrow ) : '',
			$rendered,
			$show_arrows ? str_replace( ':alt', __( 'Next', 'facetwp-carousel' ), $next_arrow ) : '',
		);
	}
}
