/*
 * This file is part of FacetWP Carousel.
 *
 * FacetWP Carousel is free software: you can redistribute it and/or
 * modify it under the terms of the GNU Affero General Public
 * License as published by the Free Software Foundation, either
 * version 3 of the License, or (at your option) any later version.
 *
 * FacetWP Carousel is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with FacetWP Carousel.  If not, see <https://www.gnu.org/licenses/>.
 */

import { find, get, isEmpty } from 'lodash';
import { __ } from '@wordpress/i18n';
import { registerBlockType } from '@wordpress/blocks';
import { PanelBody, Placeholder, SelectControl, TextControl, ToggleControl, } from '@wordpress/components';
import { InspectorControls, useBlockProps, } from '@wordpress/block-editor';
import { createReduxStore, register, useSelect } from "@wordpress/data";
import apiFetch from "@wordpress/api-fetch";
import { RawHTML } from "@wordpress/element";
import icon from './icon';

const DEFAULT_STATE = {
	templates: null,
};

const actions = {
	setTemplates( templates ) {
		return {
			type: 'SET_TEMPLATES',
			templates,
		};
	},
	fetchFromAPI( path ) {
		return {
			type: 'FETCH_FROM_API',
			path,
		};
	},
};

const store = createReduxStore( 'facetwp-carousel', {
		reducer( state = DEFAULT_STATE, action ) {
			switch ( action.type ) {
				case 'SET_TEMPLATES':
					return {
						templates: action.templates,
					};
			}
			return state;
		},
		actions,
		selectors: {
			getTemplates: ( state ) => state.templates,
		},
		controls: {
			FETCH_FROM_API: ( action ) => apiFetch( { path: action.path } ),
		},
		resolvers: {
			* getTemplates() {
				const path = '/facetwp-carousel/v1/templates/';
				const templates = yield actions.fetchFromAPI( path );
				return actions.setTemplates( templates );
			}
		}
	} )
;
register( store );

registerBlockType(
	'tandem/facetwp-carousel',
	{
		apiVersion: 2,
		title: __( 'FacetWP Carousel', 'facetwp-carousel' ),
		icon,
		category: 'widgets',
		description: __( 'Display posts as a carousel using a FacetWP template.', 'facetwp-carousel' ),
		attributes: {
			templateName: {
				type: "string",
				default: '',
			},
			useTemplateQuery: {
				type: "boolean",
				default: false
			},
			postTypeSlug: {
				type: "string",
				default: "post",
			},
			taxonomySlug: {
				type: "string",
				default: "category",
			},
			categorySlug: {
				type: "string",
				default: "",
			},
			limit: {
				type: 'number',
				default: 10,
			},
			featuredProducts: {
				type: 'bool',
				default: false,
			},
			desktopSlides: {
				type: 'number',
				default: 4,
			},
			tabletSlides: {
				type: 'number',
				default: 3,
			},
			mobileSlides: {
				type: 'number',
				default: 2,
			},
			arrows: {
				type: 'boolean',
				default: true,
			},
			dots: {
				type: 'boolean',
				default: false,
			}
		},
		supports: {
			html: false,
		},
		edit( { attributes, setAttributes } ) {
			const {
				useTemplateQuery,
				postTypeSlug,
				taxonomySlug,
				categorySlug,
				templateName,
				limit,
				desktopSlides,
				tabletSlides,
				mobileSlides,
				featuredProducts,
				arrows,
				dots,
			} = attributes;
			const { postTypes, taxonomies, categories, templates } = useSelect( ( select ) => {
					const { getPostTypes, getTaxonomies, getEntityRecords } = select( 'core' );
					const { getTemplates } = select( 'facetwp-carousel' );
					const postTypes = getPostTypes( { per_page: - 1, context: 'view' } )
					const taxonomies = getTaxonomies( { per_page: - 1, context: 'view' } );
					const categories = getEntityRecords( 'taxonomy', taxonomySlug, { per_page: - 1, context: 'view' } );
					const templates = getTemplates();
					return {
						postTypes: Array.isArray( postTypes )
							? postTypes.map( ( { slug, name } ) => (
								{
									value: slug,
									label: name,
								}
							) )
							: [],
						taxonomies: Array.isArray( taxonomies )
							? taxonomies.map( ( { slug, name, types } ) => (
								{
									value: slug,
									label: name,
									objectTypes: types,
								}
							) )
							: [],
						categories: Array.isArray( categories )
							? categories.map( ( { slug, name } ) => (
								{
									value: slug,
									label: name,
								}
							) )
							: categories,
						templates: Array.isArray( templates )
							? templates.map( ( { name, label } ) => (
								{
									value: name,
									label,
								}
							) )
							: [],
					}
				},
				[
					postTypeSlug,
					taxonomySlug,
				]
			);
			const templateSelect = <SelectControl
				label={ __( 'Template', 'facetwp-carousel' ) }
				value={ templateName }
				onChange={ ( value ) => {
					setAttributes( {
						templateName: value,
					} )
				} }
				options={ [
					{
						value: '',
						label: __( 'Select a template', 'facetwp-carousel' ),
						disabled: true
					},
					...templates,
				] }
			/>;
			return (
				<div  { ...useBlockProps() } >
					<InspectorControls>
						<PanelBody
							title={ __( 'FacetWP settings', 'facetwp-carousel' ) }>
							{ templateSelect }
							<ToggleControl
								label={ __( 'Use template query', 'facetwp-carousel' ) }
								checked={ useTemplateQuery }
								onChange={ ( value ) => setAttributes( {
									useTemplateQuery: value,
								} ) }
							/>
						</PanelBody>
						{ ! useTemplateQuery && <PanelBody
							title={ __( 'Post settings', 'facetwp-carousel' ) }>
							<SelectControl
								label={ __( 'Post Type', 'facetwp-carousel' ) }
								value={ postTypeSlug }
								options={ postTypes }
								onChange={ ( value ) => {
									setAttributes( {
										postTypeSlug: value,
										taxonomySlug: get(
											find( taxonomies, ( taxonomy ) => taxonomy.objectTypes.includes( value ) ),
											'value',
											''
										),
										categorySlug: '',
										featuredProducts: value === 'product' ? featuredProducts : false,
									} );
								} }
							/>
							<SelectControl
								label={ __( 'Taxonomy', 'facetwp-carousel' ) }
								value={ taxonomySlug }
								options={ taxonomies.filter( taxonomy => taxonomy.objectTypes.includes( postTypeSlug ) ) }
								onChange={ ( value ) => {
									setAttributes( {
										taxonomySlug: value,
										categorySlug: '',
									} );
								} }
							/>
							{ ! isEmpty( categories ) && (
								<SelectControl
									label={ __( 'Category', 'facetwp-carousel' ) }
									value={ categorySlug }
									options={ [
										{
											value: '',
											label: __( 'All Categories', 'facetwp-carousel' ),
										},
										...categories
									] }
									onChange={ ( value ) => {
										setAttributes( {
											categorySlug: value,
										} );
									} }
								/>
							) }
						</PanelBody> }
						{ ! useTemplateQuery && <PanelBody
							title={ __( 'Filter and Sorting', 'facetwp-carousel' ) }
						>
							<TextControl
								label={ __( 'Max. posts', 'facetwp-carousel' ) }
								type="number"
								min="-1"
								value={ limit }
								onChange={ ( value ) => setAttributes( {
									limit: parseInt( value, 10 ),
								} ) }
							/>
							{ postTypeSlug === 'product' && <ToggleControl
								label={ __( 'Only featured products', 'facetwp-carousel' ) }
								checked={ featuredProducts }
								onChange={ ( value ) => setAttributes( {
									featuredProducts: value,
								} ) }
							/> }
						</PanelBody> }
						<PanelBody
							title={ __( 'Slides', 'facetwp-carousel' ) }
						>
							<TextControl
								label={ __( 'Desktop', 'facetwp-carousel' ) }
								type="number"
								min="1"
								value={ desktopSlides }
								onChange={ ( value ) => setAttributes( {
									desktopSlides: parseInt( value, 10 ),
								} ) }
							/>
							<TextControl
								label={ __( 'Tablet', 'facetwp-carousel' ) }
								type="number"
								min="1"
								value={ tabletSlides }
								onChange={ ( value ) => setAttributes( {
									tabletSlides: parseInt( value, 10 ),
								} ) }
							/>
							<TextControl
								label={ __( 'Mobile', 'facetwp-carousel' ) }
								type="number"
								min="1"
								value={ mobileSlides }
								onChange={ ( value ) => setAttributes( {
									mobileSlides: parseInt( value, 10 ),
								} ) }
							/>
						</PanelBody>
						<PanelBody
							title={ __( 'Style', 'facetwp-carousel' ) }
						>
							<ToggleControl
								label={ __( 'Show arrows', 'facetwp-carousel' ) }
								checked={ arrows }
								onChange={ ( value ) => setAttributes( {
									arrows: value,
								} ) }
							/>
							<ToggleControl
								label={ __( 'Show dots', 'facetwp-carousel' ) }
								checked={ dots }
								onChange={ ( value ) => setAttributes( {
									dots: value,
								} ) }
							/>
						</PanelBody>
					</InspectorControls>
					<Placeholder icon={ icon } label={ __( 'FacetWP Carousel', 'facetwp-carousel' ) }>
						{ templateSelect }
					</Placeholder>
				</div>
			);
		},
		save( { attributes } ) {
			const {
				templateName,
				useTemplateQuery,
				postTypeSlug,
				taxonomySlug,
				categorySlug,
				limit,
				desktopSlides,
				tabletSlides,
				mobileSlides,
				featuredProducts,
				arrows,
				dots,
			} = attributes;
			return <RawHTML>{ '[facetwp_carousel'
				+ ` template="${ templateName }"`
				+ ` post_type="${ postTypeSlug }"`
				+ ` taxonomy="${ taxonomySlug }"`
				+ ` category="${ categorySlug || '' }"`
				+ ` limit=${ limit }`
				+ (
					featuredProducts ? ` featured_products=1` : ''
				)
				+ ` slides_desktop=${ desktopSlides }`
				+ ` slides_tablet=${ tabletSlides }`
				+ ` slides_mobile=${ mobileSlides }`
				+ (
					arrows ? '' : ' arrows=0'
				)
				+ (
					dots ? ' dots=1' : ''
				)
				+ (
					useTemplateQuery ? ' use_template_query=1' : ''
				)
				+ ']' }</RawHTML>;
		},
	}
);
